#include <stdio.h>
#include <math.h>

void Omega(double *omega_in, double *divergence, double *u_wind, double *v_wind,
	   double *halfpress, double *fullpress, double *dpsdx, double *dpsdy,
	   double *vct, int dimgp, int dim3gp, int nlev)
{
  int i, j;
  double DeltaHybrid, Cterm, Pterm;
  double *diver, *halfp, *fullp, *uwind, *vwind;
  double *omega = omega_in;

  /* Compute half level part of vertical velocity */

  for ( i = 0; i < dimgp; i++ ) omega[i] = 0.0;

  for ( j = 0; j < nlev; j++ )
    {
      omega = omega_in   + j*dimgp;
      halfp = halfpress  + j*dimgp;
      diver = divergence + j*dimgp;
      uwind = u_wind     + j*dimgp;
      vwind = v_wind     + j*dimgp;

      DeltaHybrid = vct[nlev+j+2] - vct[nlev+j+1];
#if defined (SX)
#pragma vdir nodep
#endif
#if defined (__uxp__)
#pragma loop novrec
#endif
#if defined (_OPENMP)
#pragma omp parallel for
#endif
      for ( i = 0; i < dimgp; i++ )
	{
	  omega[i+dimgp] = omega[i]
	                 - diver[i] * (halfp[i+dimgp] - halfp[i])
	                 - DeltaHybrid * (uwind[i]*dpsdx[i] + vwind[i]*dpsdy[i]);
	}
    }

  /* interpolate to full levels  */

  for ( j = 0; j < nlev; j++ )
    {
      omega = omega_in   + j*dimgp;
#if defined (SX)
#pragma vdir nodep
#endif
#if defined (__uxp__)
#pragma loop novrec
#endif
#if defined (_OPENMP)
#pragma omp parallel for
#endif
      for ( i = 0; i < dimgp; i++ )
	omega[i] = 0.5 * (omega[i] + omega[i+dimgp]);
    }

  /* compute full level part of vertical velocity */

#if defined (_OPENMP)
#pragma omp parallel for default(shared) private(i, omega, halfp, fullp, uwind, vwind, DeltaHybrid, Cterm, Pterm)
#endif
  for ( j = 0; j < nlev; j++ )
    {
      omega = omega_in   + j*dimgp;
      halfp = halfpress  + j*dimgp;
      fullp = fullpress  + j*dimgp;
      uwind = u_wind     + j*dimgp;
      vwind = v_wind     + j*dimgp;

      DeltaHybrid = vct[nlev+j+2] - vct[nlev+j+1];
      if ( DeltaHybrid )
	{
	  Cterm = vct[j+1]*vct[nlev+j+1] - vct[j]*vct[nlev+j+2];
#if defined (__uxp__)
#pragma loop novrec
#endif
	  for ( i = 0; i < dimgp; i++ )
	    {
	      if ( Cterm != 0.0 )
		Pterm = Cterm / (halfp[i+dimgp]-halfp[i]) * log(halfp[i+dimgp]/halfp[i]);
	      else
		Pterm = 0.0;

	      omega[i] += fullp[i] * (uwind[i]*dpsdx[i] + vwind[i]*dpsdy[i])
	 	        / (halfp[i+dimgp] - halfp[i]) * (DeltaHybrid + Pterm);
	    }
	}
    }
}
